diff --git a/sys/dev/disk/ahci/ahci.c b/sys/dev/disk/ahci/ahci.c
index 3cfc755..1d4e9fe 100644
--- a/sys/dev/disk/ahci/ahci.c
+++ b/sys/dev/disk/ahci/ahci.c
@@ -127,20 +127,30 @@ ahci_init(struct ahci_softc *sc)
 	 *	Power cycling the PM has no effect (it works fine on another
 	 *	host port).  This issue is unrelated to CLO.
 	 */
-#if 0
-	ahci_wait_ne(sc, AHCI_REG_GHC, AHCI_REG_GHC_HR, AHCI_REG_GHC_HR);
-#endif
+	/*
+	 * Wait for any prior reset sequence to complete
+	 */
+	if (ahci_wait_ne(sc, AHCI_REG_GHC,
+			 AHCI_REG_GHC_HR, AHCI_REG_GHC_HR) != 0) {
+		device_printf(sc->sc_dev, "Controller is stuck in reset\n");
+		return (1);
+	}
 	ahci_write(sc, AHCI_REG_GHC, AHCI_REG_GHC_AE);
+	ahci_os_sleep(500);
 	ahci_read(sc, AHCI_REG_GHC);		/* flush */
 	ahci_write(sc, AHCI_REG_GHC, AHCI_REG_GHC_AE | AHCI_REG_GHC_HR);
+	ahci_os_sleep(500);
 	ahci_read(sc, AHCI_REG_GHC);		/* flush */
 	if (ahci_wait_ne(sc, AHCI_REG_GHC,
 			 AHCI_REG_GHC_HR, AHCI_REG_GHC_HR) != 0) {
-		device_printf(sc->sc_dev,
-			      "unable to reset controller\n");
+		device_printf(sc->sc_dev, "unable to reset controller\n");
 		return (1);
 	}
-	ahci_os_sleep(100);
+	if (ahci_read(sc, AHCI_REG_GHC) & AHCI_REG_GHC_AE) {
+		device_printf(sc->sc_dev, "AE did not auto-clear!\n");
+		ahci_write(sc, AHCI_REG_GHC, 0);
+		ahci_os_sleep(500);
+	}
 
 	/*
 	 * Enable ahci (global interrupts disabled)
@@ -148,6 +158,7 @@ ahci_init(struct ahci_softc *sc)
 	 * Restore saved parameters.  Avoid pci transaction burst write
 	 * by issuing dummy reads.
 	 */
+	ahci_os_sleep(500);
 	ahci_write(sc, AHCI_REG_GHC, AHCI_REG_GHC_AE);
 	ahci_os_sleep(500);
 
